use PartitioningDb
go

ALTER DATABASE [PartitioningDb] ADD FILEGROUP [DATA2013]
GO
ALTER DATABASE [PartitioningDb] ADD FILE ( NAME = N'PartitioningDB_2013', FILENAME = N'c:\db\PartitioningDB_2013.ndf' , SIZE = 3072KB , FILEGROWTH = 10240KB ) TO FILEGROUP [DATA2013]
GO

create partition function pfOrders2013(datetime2(0))
as range right for values
(
	'2013-02-01','2013-03-01','2013-04-01','2013-05-01'
	,'2013-06-01','2013-07-01','2013-08-01','2013-09-01'
	,'2013-10-01','2013-11-01','2013-12-01'
)
go

create partition scheme psOrders2013
as partition pfOrders2013
all to ([Data2013])
go

create table dbo.Orders2013
(
	OrderId int not null,
	OrderDate datetime2(0) not null,
	OrderNum varchar(32) not null,
	Placeholder char(100) null,
	
	--constraint CHK_Orders2013
	--check(OrderDate >= '2013-01-01' and OrderDate < '2013-12-01')
)
on [Data2013]
go

create unique clustered index IDX_Orders2013_OrderDate_OrderID
on dbo.Orders2013(OrderDate,OrderId)
with (data_compression=page)
on psOrders2013(OrderDate)
go

create unique nonclustered index IDX_Orders2013_OrderNum_OrderDate
on dbo.Orders2013(OrderNum,OrderDate)
with (data_compression=page)
on psOrders2013(OrderDate)
go

-- Creating Jan 2013
create table dbo.Orders2013_01
(
	OrderId int not null,
	OrderDate datetime2(0) not null,
	OrderNum varchar(32) not null,
	Placeholder char(100) null,
	
	constraint CHK_Orders2013_01
	check(OrderDate >= '2013-01-01' and OrderDate < '2013-02-01')
)
on [FASTSTORAGE]
go

create unique clustered index IDX_Orders2013_01_OrderDate_OrderID
on dbo.Orders2013_01(OrderDate,OrderId)
on [FASTSTORAGE]
go

create unique nonclustered index IDX_Orders2013_01_OrderNum_OrderDate
on dbo.Orders2013_01(OrderNum,OrderDate)
on [FASTSTORAGE]
go

-- Moving Dec 2012
create unique nonclustered index IDX_Orders2012_12_OrderNum_OrderDate
on dbo.Orders2012_12(OrderNum,OrderDate)
with (data_compression = page, online = on, drop_existing = on)
on [Data2012]
go

create unique clustered index IDX_Orders2012_12_OrderDate_OrderID
on dbo.Orders2012_12(OrderDate,OrderId)
with (data_compression = page, online = on, drop_existing = on)
on [Data2012]
go

begin tran 
	alter table dbo.Orders2012
	add	constraint CHK_Orders2012_Final
	check
		(OrderDate >= '2012-01-01' and OrderDate < '2013-01-01')
		
	drop view dbo.Orders
	
	alter table dbo.Orders2012
	drop constraint CHK_Orders2012

	alter table dbo.Orders2012_12 
	switch to dbo.Orders2012 partition 12

	exec sp_executesql N'	
create view dbo.Orders(OrderDate, OrderId, OrderNum, Placeholder)
as
	select OrderDate, OrderId, OrderNum, PlaceHolder
	from dbo.Orders2011

	union all 

	select OrderDate, OrderId, OrderNum, PlaceHolder
	from dbo.Orders2012

	union all 

	select OrderDate, OrderId, OrderNum, PlaceHolder
	from dbo.Orders2013_01'

	drop table dbo.Orders2012_12			    
commit
go