/*************************************************************************************************
Dmitri Korotkevitch
24HOP - Data partitioning in VLDB (second edition)

http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

Partitioned Table - creation script
(SQL Server 2012)
*************************************************************************************************/

--select @@VERSION

set nocount on
set xact_abort on
go

use Partitioning2012
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'Data1'
)
	drop table dbo.Data1
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'TmpData1'
)
	drop table dbo.TmpData1
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = '_Data1'
)
	drop table dbo._Data1
go

if exists
(
	select * 
	from sys.views v join sys.schemas s on v.schema_id = s.schema_id
	where s.name = 'dbo' and v.name = 'Data1'
)
	drop view dbo.Data1
go

if exists
(
	select * 
	from sys.partition_schemes
	where name = 'psData1'
)
	drop partition scheme psData1
go

if exists
(
	select * 
	from sys.partition_schemes
	where name = 'psData1_2'
)
	drop partition scheme psData1_2
go


if exists
(
	select * 
	from sys.partition_schemes
	where name = 'psTmpData'
)
	drop partition scheme psTmpData
go

if exists
(
	select * 
	from sys.partition_functions
	where name = 'pfData1'
)
	drop partition function pfData1
go

if exists
(
	select * 
	from sys.partition_functions
	where name = 'pfTmpData'
)
	drop partition function pfTmpData
go



create partition function pfData1(date)
as range right for values 
('2013-01-01','2013-02-01','2013-03-01','2013-04-01'
,'2013-05-01','2013-06-01','2013-07-01','2013-08-01'
,'2013-09-01')
go

create partition scheme psData1
as partition pfData1
all to ([FG1])
go

create table dbo.Data1
(
	OrderDate date not null,
	OrderId int not null identity(1,1),
	OrderNum varchar(32) not null,
	LobColumn varchar(max) null,
	Placeholder char(50) null,
)
go

create unique clustered index IDX_Data1_OrderDate_OrderId
on dbo.Data1(OrderDate, OrderId)
on psData1(OrderDate)
go

declare
	@StartDate datetime = '2012-12-01'

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows
,IDs(ID) as (select ROW_NUMBER() over (order by (select NULL)) from N4)
insert into dbo.Data1(OrderDate, OrderNum)
	select 
		dateadd(day,[days].ID,@StartDate), 
		'Order: ' + convert(varchar(12),dateadd(day,[days].ID,@StartDate)) + '-' + convert(varchar(5),[orders].Id)
	from IDs [days] cross join IDs [orders]
go


select Partitions.Number, Partitions.Value, Cnt.Cnt
from 
	(
		-- Does not include right-most partition
		select prv.boundary_id, prv.value
		from sys.partition_range_values prv	join sys.partition_functions pf on
			pf.function_id = prv.function_id
		where 
			pf.name = 'pfData1'
	) Partitions(Number, Value)
	cross apply
	(
		select count(*) as Cnt
		from dbo.Data1
		where $Partition.pfData1(OrderDate) = Partitions.Number
	) cnt
order by 
	Partitions.Number
go
	
