/*************************************************************************************************
Dmitri Korotkevitch
24HOP - Data partitioning in VLDB (second edition)

http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

Partitioned Table - moving data scripts (2)
(SQL Server 2012)
*************************************************************************************************/


set nocount on
set xact_abort on
go

use Partitioning2012
go


-- ??? ????:
--	1. ??????? ????????? ??????? ? ??????????? ? ??? ?????? (????????)
--	2. ????????????? ???????? ??????? ? ??????? ???????????????? ????????????? (????????)
--	3. ????????? ????????? ??????? ?? ?????? ???????? ?????? (online index rebuild) 
--	4. ????????? ????????? ??????? ??? ?????? ? ???????? ??????? (????????)
--	5. ??????? ???????????????? ????????????? ? ????????????? ???????? ??????? (????????)

-- 1 - 2
create table dbo.TmpData1
(
	OrderDate date not null,
	OrderId int not null identity(1,1),
	OrderNum varchar(32) not null,
	LobColumn varchar(max) null,
	Placeholder char(50) null,

	constraint CHK_TmpData1
	check (OrderDate < '2013-01-01')
)
TEXTIMAGE_ON [FG1]
go

create unique clustered index IDX_Data1_OrderDate_OrderId
on dbo.TmpData1(OrderDate, OrderId)
on [FG1]
go

begin tran
	alter table dbo.Data1 switch partition 1 to dbo.TmpData1 -- Metadata with Schema-M lock at this point
	
	alter table dbo.Data1 
		add constraint CHK_Data1_MinDate
		check(OrderDate >= '2013-01-01')

	alter partition function pfData1()
	merge range ('2013-01-01')

	exec sp_rename 'dbo.Data1', '_Data1', 'OBJECT'

	exec sp_executesql N'
	create view dbo.Data1(OrderId, OrderDate, OrderNum, PlaceHolder)
	as
		select OrderId, OrderDate, OrderNum, PlaceHolder
		from dbo._Data1

		union all 

		select OrderId, OrderDate, OrderNum, PlaceHolder
		from dbo.TmpData1
	'
commit
go

-- Enable "Display execution plan"
select top 10 * 
from dbo.Data1 
where OrderDate < '2013-01-01'
order by OrderDate
go


-- 3
create partition function pfTmpData(date)
as range right for values 
('2014-01-01')
go

create partition scheme psTmpData
as partition pfTmpData
all to ([FG2])
go

create unique clustered index IDX_Data1_OrderDate_OrderId
on dbo.TmpData1(OrderDate, OrderId)
with (drop_existing=on, online=on)
on psTmpData(OrderDate)
go

alter partition function pfTmpData()
merge range('2014-01-01')
go


-- 4
alter partition scheme psData1
next used [FG1]
go

alter partition function pfData1() split range('2013-01-01') -- metadata
go

alter partition scheme psData1
next used [FG2]
go

alter partition function pfData1() split range('2012-01-01') -- metadata
go


alter partition function pfData1() merge range ('2012-01-01')
go

begin tran
	alter table dbo._Data1 
	drop constraint CHK_Data1_MinDate

	alter table dbo.TmpData1 switch partition 1 to dbo._Data1 partition 1 

	drop view dbo.Data1

	exec sp_rename 'dbo.Data1', '_Data1', 'OBJECT'

	drop table dbo.TmpData1
	drop partition scheme psTmpData
	drop partition function pfTmpData
commit
go	
